/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
 */

/* 
 * GtkSegment -- An LED segment display widget
 * Original Code: Syd Logan 9/21/2001
 */

#include <stdio.h>
#include "gtksegment.h"

#define MIN_SEGMENT_SIZE 16

static void gtk_segment_class_init(GtkSegmentClass *klass);
static void gtk_segment_init(GtkSegment *segment);
static void gtk_segment_unrealize(GtkWidget *widget);
static gint gtk_segment_expose(GtkWidget *widget, GdkEventExpose *event);
static void gtk_segment_size_request(GtkWidget *widget, GtkRequisition *requisition);
static void gtk_segment_size_allocate(GtkWidget *widget, GtkAllocation *allocation);
static void draw_segment_dot( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, int height, int x, int y, int extent );
static void draw_segment_one( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, int height, int x, int y, int extent );
static void draw_segment_two( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, int height, int x, int y, int extent );
static void draw_segment_three( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, int height, int x, int y, int extent );
static void draw_segment_four( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, int height, int x, int y, int extent );
static void draw_segment_five( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, int height, int x, int y, int extent );
static void draw_segment_six( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, int height, int x, int y, int extent );
static void draw_segment_seven( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, int height, int x, int y, int extent );
static void clear_segments( GdkWindow *window, int x, int y, int extent ); 

GtkType
gtk_segment_get_type (void)
{
  static GtkType segment_type = 0;

  if (!segment_type)
    {
      static const GtkTypeInfo segment_info =
      {
	      "GtkSegment",
	      sizeof (GtkSegment),
	      sizeof (GtkSegmentClass),
	      (GtkClassInitFunc) gtk_segment_class_init,
	      (GtkObjectInitFunc) gtk_segment_init,
	      /* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };
      segment_type = gtk_type_unique (GTK_TYPE_MISC, &segment_info);
    }

  return segment_type;
}

static void
gtk_segment_class_init (GtkSegmentClass *class)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;

/*
  gtk_object_add_arg_type ("GtkSegment::segment_color", GTK_TYPE_SEGMENT_COLOR, GTK_ARG_READWRITE, ARG_SEGMENT_COLOR);

  object_class->set_arg = gtk_segment_set_arg;
  object_class->get_arg = gtk_segment_get_arg;
*/

  widget_class->expose_event = gtk_segment_expose;
  widget_class->size_request = gtk_segment_size_request;
  widget_class->unrealize = gtk_segment_unrealize;
}

/*
static void
gtk_segment_set_arg (GtkObject      *object,
		   GtkArg         *arg,
		   guint           arg_id)
{
  GtkSegment *segment;

  segment = GTK_SEGMENT (object);

  switch (arg_id)
    {
    case ARG_SEGMENT_COLOR:
      break;
    default:
      break;
    }
}

static void
gtk_segment_get_arg (GtkObject      *object,
		   GtkArg         *arg,
		   guint           arg_id)
{
  GtkSegment *segment;

  segment = GTK_SEGMENT (object);
  switch (arg_id)
    {
    case ARG_SEGMENT_COLOR:
      GTK_VALUE_ENUM (*arg) = segment->segment_color;
      break;
    default:
      arg->type = GTK_TYPE_INVALID;
      break;
    }
}

*/

static void
gtk_segment_init (GtkSegment *segment)
{
  GTK_WIDGET_SET_FLAGS (segment, GTK_NO_WINDOW);

  segment->segment_color = GTK_SEGMENT_BLACK;
  segment->segment_value = '*';   // i.e., undefined
  segment->redGC = (GdkGC *) NULL;
  segment->greenGC = (GdkGC *) NULL;
  segment->blueGC = (GdkGC *) NULL;
  segment->yellowGC = (GdkGC *) NULL;
}

GtkWidget*
gtk_segment_new()
{
  GtkSegment *segment;

  segment = gtk_type_new (GTK_TYPE_SEGMENT);

  return GTK_WIDGET (segment);
}

static void
gtk_segment_unrealize (GtkWidget *widget)
{
  GtkSegment *segment = GTK_SEGMENT(widget);
  if ( segment->greenGC != (GdkGC *) NULL ) {
    gdk_gc_destroy( segment->greenGC );
    segment->greenGC = (GdkGC *) NULL;
  }
  if ( segment->blueGC != (GdkGC *) NULL ) {
    gdk_gc_destroy( segment->blueGC );
    segment->blueGC = (GdkGC *) NULL;
  }
  if ( segment->redGC != (GdkGC *) NULL ) {
    gdk_gc_destroy( segment->redGC );
    segment->redGC = (GdkGC *) NULL;
  }
  if ( segment->yellowGC != (GdkGC *) NULL ) {
    gdk_gc_destroy( segment->yellowGC );
    segment->yellowGC = (GdkGC *) NULL;
  }
}

GtkWidget*
gtk_segment_new_with_color(GtkSegmentColor color)
{
  GtkSegment *segment;

  segment = gtk_type_new (GTK_TYPE_SEGMENT);

  segment->segment_color = color;

  return GTK_WIDGET (segment);
}

int
gtk_segment_set_color(GtkSegment *segment, GtkSegmentColor segment_color)
{
  int ret = FALSE;
  g_return_if_fail (segment != NULL);
  g_return_if_fail (GTK_IS_SEGMENT (segment));

  if (((GtkSegmentColor) segment->segment_color != segment_color))
    {
      ret = TRUE;
      segment->segment_color = segment_color;

      if (GTK_WIDGET_DRAWABLE (segment))
	      gtk_widget_queue_clear (GTK_WIDGET (segment));
    }
  return( ret );
}

int
gtk_segment_set_value(GtkSegment *segment, unsigned char value)
{
  int ret = FALSE;
  g_return_if_fail (segment != NULL);
  g_return_if_fail (GTK_IS_SEGMENT (segment));

  if ( value == '.' || value == '-' || 
    ( value >= '0' && value <= '9' ) || value == 'A' || value == 'b' || 
    value == 'C' || value == 'd' || value == 'E' || value == 'F' )
      
    {
      ret = TRUE;
      segment->segment_value = value;

      if (GTK_WIDGET_DRAWABLE (segment))
	      gtk_widget_queue_clear (GTK_WIDGET (segment));
    }
  return( ret );
}

static void 
gtk_segment_size_request(GtkWidget *widget, GtkRequisition *requisition)
{
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_SEGMENT (widget));
  g_return_if_fail (requisition != NULL);

  if ( requisition->height < MIN_SEGMENT_SIZE )
    requisition->height = MIN_SEGMENT_SIZE;
  if ( requisition->width < MIN_SEGMENT_SIZE )
    requisition->width = MIN_SEGMENT_SIZE;
  if ( GTK_SEGMENT(widget)->segment_value == '.' )
    requisition->width = requisition->width >> 1;
}

static gint
gtk_segment_expose (GtkWidget *widget, GdkEventExpose *event)
{
  GtkSegment *segment;
  gint width, height;
  gint x, y;
  gint extent;
  GdkGC *fg = widget->style->fg_gc[GTK_STATE_NORMAL];
  GdkGC *bg = widget->style->bg_gc[GTK_STATE_NORMAL];

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_SEGMENT (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (GTK_WIDGET_DRAWABLE (widget)) 
  {
    GdkColormap *colormap;
    GdkGCValues values; 

    colormap = gtk_widget_get_colormap( widget );
    segment = GTK_SEGMENT (widget);

    if ( segment->segment_color == GTK_SEGMENT_RED ) {
      if ( segment->redGC == (GdkGC *) NULL ) {
        values.background = widget->style->base[GTK_STATE_NORMAL];
        values.foreground.red = 65535;
        values.foreground.green = 0;
        values.foreground.blue = 0;
        gdk_color_alloc(colormap, &values.foreground);
        segment->redGC = gdk_gc_new_with_values(widget->window, &values,
          GDK_GC_FOREGROUND | GDK_GC_BACKGROUND );
      }
      fg = segment->redGC;
    }
    else if ( segment->segment_color == GTK_SEGMENT_GREEN ) {
      if ( segment->greenGC == (GdkGC *) NULL ) {
        values.background = widget->style->base[GTK_STATE_NORMAL];
        values.foreground.red = 0;
        values.foreground.green = 65535;
        values.foreground.blue = 0;
        gdk_color_alloc(colormap, &values.foreground);
        segment->greenGC = gdk_gc_new_with_values(widget->window, &values,
          GDK_GC_FOREGROUND | GDK_GC_BACKGROUND );
      }
      fg = segment->greenGC;
    }
    else if ( segment->segment_color == GTK_SEGMENT_BLUE ) {
      if ( segment->blueGC == (GdkGC *) NULL ) {
        values.background = widget->style->base[GTK_STATE_NORMAL];
        values.foreground.red = 0;
        values.foreground.green = 0;
        values.foreground.blue = 65535;
        gdk_color_alloc(colormap, &values.foreground);
        segment->blueGC = gdk_gc_new_with_values(widget->window, &values,
          GDK_GC_FOREGROUND | GDK_GC_BACKGROUND );
      }
      fg = segment->blueGC;
    }
    else if ( segment->segment_color == GTK_SEGMENT_YELLOW ) {
      if ( segment->yellowGC == (GdkGC *) NULL ) {
        values.background = widget->style->base[GTK_STATE_NORMAL];
        values.foreground.red = 65535;
        values.foreground.green = 65535;
        values.foreground.blue = 0;
        gdk_color_alloc(colormap, &values.foreground);
        segment->yellowGC = gdk_gc_new_with_values(widget->window, &values,
          GDK_GC_FOREGROUND | GDK_GC_BACKGROUND );
      }
      fg = segment->yellowGC;
    }

    width = widget->allocation.width; 
    height = widget->allocation.height;
    extent = MIN (width, height);

    x = widget->allocation.x;
    y = widget->allocation.y;

    gdk_gc_set_clip_rectangle (fg, &event->area);
    gdk_gc_set_clip_rectangle (bg, &event->area);
 
    clear_segments( widget->window, x, y, extent ); 

    switch (segment->segment_value)
    {
    case '.':
      draw_segment_dot( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '-':
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '0':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '1':
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '2':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '3':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '4':
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '5':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '6':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '7':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '8':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case '9':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case 'A':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case 'b':
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case 'C':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case 'd':
      draw_segment_three( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_six( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case 'E':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_seven( fg, bg, widget->window, width, height, x, y, extent );
      break;
    case 'F':
      draw_segment_one( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_two( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_four( fg, bg, widget->window, width, height, x, y, extent );
      draw_segment_five( fg, bg, widget->window, width, height, x, y, extent );
      break;
    default:  // impossible
      break;
    }

    gdk_gc_set_clip_rectangle (fg, NULL);
    gdk_gc_set_clip_rectangle (bg, NULL);
  }
  return TRUE;
}

static void
clear_segments( GdkWindow *window, int x, int y, int extent ) 
{
  gdk_window_clear_area(window, x, y, extent, extent); 
}

static void
draw_segment_dot( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, 
  int height, int x, int y, int extent ) 
{
  int wide, h_offset, margin, segwide;

  width = width << 1;
  wide = 0.4 * width;

  segwide = wide * 0.4;

  h_offset = (width - (wide + margin * 2)) / 2;

  gdk_draw_arc( (GdkDrawable *) window, fg, TRUE,
    x + segwide, y + (height - segwide), segwide, segwide, 0, 23040 );
}

static void
draw_segment_one( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width, 
  int height, int x, int y, int extent ) 
{
  int wide, h_offset, margin, segwide;
  GdkPoint points[7];

  wide = 0.6 * width;

  segwide = wide * 0.25;

  margin = segwide / 2;

  h_offset = (width - (wide + margin * 2)) / 2;

  points[0].x = h_offset + margin - margin;
  points[0].x += x;
  points[0].y = 0;
  points[0].y += y;
  points[1].x = width - (h_offset + margin) - margin;
  points[1].x += x;
  points[1].y = 0;
  points[1].y += y;
  points[2].x = width - h_offset - (segwide + margin) - margin;
  points[2].x += x;
  points[2].y = segwide;
  points[2].y += y;
  points[3].x = h_offset + segwide + margin - margin;
  points[3].x += x;
  points[3].y = segwide;
  points[3].y += y;
  points[4].x = h_offset + margin - margin;
  points[4].x += x;
  points[4].y = 0;
  points[4].y += y;

  gdk_draw_polygon( (GdkDrawable *) window, fg, TRUE,
    &points[0], 5 );
}

static void
draw_segment_two( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width,
  int height, int x, int y, int extent ) 
{
  int wide, h_offset, margin, segwide;
  GdkPoint points[7];

  wide = 0.6 * width;

  segwide = wide * 0.25;

  margin = segwide / 2;

  h_offset = (width - (wide + margin * 2)) / 2;

  points[0].x = h_offset - margin;
  points[0].x += x;
  points[0].y = 0;
  points[0].y += y;
  points[1].x = h_offset + segwide - margin;
  points[1].x += x;
  points[1].y = segwide;
  points[1].y += y;
  points[2].x = h_offset + segwide - margin;
  points[2].x += x;
  points[2].y = height / 2 - segwide;
  points[2].y += y;
  points[3].x = h_offset - margin;
  points[3].x += x;
  points[3].y = height / 2;
  points[3].y += y;
  points[4].x = h_offset - margin;
  points[4].x += x;
  points[4].y = 0;
  points[4].y += y;

  gdk_draw_polygon( (GdkDrawable *) window, fg, TRUE,
    &points[0], 5 );
}

static void
draw_segment_three( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width,
  int height, int x, int y, int extent ) 
{
  int wide, h_offset, margin, segwide;
  GdkPoint points[7];

  wide = 0.6 * width;

  segwide = wide * 0.25;

  margin = segwide / 2;

  h_offset = (width - (wide + margin * 2)) / 2;

  points[0].x = width - h_offset - margin;
  points[0].x += x;
  points[0].y = 0;
  points[0].y += y;
  points[1].x = width - h_offset - segwide - margin;
  points[1].x += x;
  points[1].y = segwide;
  points[1].y += y;
  points[2].x = width - h_offset - segwide - margin;
  points[2].x += x;
  points[2].y = height / 2 - segwide;
  points[2].y += y;
  points[3].x = width - h_offset - margin;
  points[3].x += x;
  points[3].y = height / 2;
  points[3].y += y;
  points[4].x = width - h_offset - margin;
  points[4].x += x;
  points[4].y = 0;
  points[4].y += y;

  gdk_draw_polygon( (GdkDrawable *) window, fg, TRUE,
    &points[0], 5 );
}

static void
draw_segment_four( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width,
  int height, int x, int y, int extent ) 
{
  int wide, h_offset, margin, segwide;
  GdkPoint points[7];

  wide = 0.6 * width;

  segwide = wide * 0.25;

  margin = segwide / 2;

  h_offset = (width - (wide + margin * 2)) / 2;

  points[0].x = h_offset + margin - margin;
  points[0].x += x;
  points[0].y = height / 2;
  points[0].y += y;
  points[1].x = h_offset + segwide - margin;
  points[1].x += x;
  points[1].y = height / 2 - margin;
  points[1].y += y;
  points[2].x = width - h_offset - segwide - margin;
  points[2].x += x;
  points[2].y = height / 2 - margin;
  points[2].y += y;
  points[3].x = width - h_offset - margin - margin;
  points[3].x += x;
  points[3].y = height / 2;
  points[3].y += y;
  points[4].x = width - h_offset - segwide - margin;
  points[4].x += x;
  points[4].y = height / 2 + margin;
  points[4].y += y;
  points[5].x = h_offset + segwide - margin;
  points[5].x += x;
  points[5].y = height / 2 + margin;
  points[5].y += y;
  points[6].x = h_offset + margin - margin;
  points[6].x += x;
  points[6].y = height / 2;
  points[6].y += y;

  gdk_draw_polygon( (GdkDrawable *) window, fg, TRUE,
    &points[0], 7 );
}

static void
draw_segment_five( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width,
  int height, int x, int y, int extent ) 
{
  int wide, h_offset, margin, segwide;
  GdkPoint points[7];

  wide = 0.6 * width;

  segwide = wide * 0.25;

  margin = segwide / 2;

  h_offset = (width - (wide + margin * 2)) / 2;

  points[0].x = h_offset - margin;
  points[0].x += x;
  points[0].y = height / 2;
  points[0].y += y;
  points[1].x = h_offset + segwide - margin;
  points[1].x += x;
  points[1].y = height / 2 + segwide;
  points[1].y += y;
  points[2].x = h_offset + segwide - margin;
  points[2].x += x;
  points[2].y = height - segwide;
  points[2].y += y;
  points[3].x = h_offset - margin;
  points[3].x += x;
  points[3].y = height;
  points[3].y += y;
  points[4].x = h_offset - margin;
  points[4].x += x;
  points[4].y = height / 2;
  points[4].y += y;

  gdk_draw_polygon( (GdkDrawable *) window, fg, TRUE,
    &points[0], 5 );
}

static void
draw_segment_six( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width,
  int height, int x, int y, int extent ) 
{
  int wide, h_offset, margin, segwide;
  GdkPoint points[7];

  wide = 0.6 * width;

  segwide = wide * 0.25;

  margin = segwide / 2;

  h_offset = (width - (wide + margin * 2)) / 2;

  points[0].x = width - h_offset - margin;
  points[0].x += x;
  points[0].y = height / 2;
  points[0].y += y;
  points[1].x = width - h_offset - segwide - margin;
  points[1].x += x;
  points[1].y = height / 2 + segwide;
  points[1].y += y;
  points[2].x = width - h_offset - segwide - margin;
  points[2].x += x;
  points[2].y = height - segwide;
  points[2].y += y;
  points[3].x = width - h_offset - margin;
  points[3].x += x;
  points[3].y = height;
  points[3].y += y;
  points[4].x = width - h_offset - margin;
  points[4].x += x;
  points[4].y = height / 2;
  points[4].y += y;

  gdk_draw_polygon( (GdkDrawable *) window, fg, TRUE,
    &points[0], 5 );
}

static void
draw_segment_seven( GdkGC *fg, GdkGC *bg, GdkWindow *window, int width,
  int height, int x, int y, int extent ) 
{
  int wide, h_offset, margin, segwide;
  GdkPoint points[7];

  wide = 0.6 * width;

  segwide = wide * 0.25;

  margin = segwide / 2;

  h_offset = (width - (wide + margin * 2)) / 2;

  points[0].x = h_offset + margin - margin;
  points[0].x += x;
  points[0].y = height;
  points[0].y += y;
  points[1].x = width - (h_offset + margin) - margin;
  points[1].x += x;
  points[1].y = height;
  points[1].y += y;
  points[2].x = width - h_offset - (segwide + margin) - margin;
  points[2].x += x;
  points[2].y = height - segwide;
  points[2].y += y;
  points[3].x = h_offset + segwide + margin - margin;
  points[3].x += x;
  points[3].y = height - segwide;
  points[3].y += y;
  points[4].x = h_offset + margin - margin;
  points[4].x += x;
  points[4].y = height;
  points[4].y += y;

  gdk_draw_polygon( (GdkDrawable *) window, fg, TRUE,
    &points[0], 5 );
}

