/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
 */

#ifndef __GTK_SEGMENT_H__
#define __GTK_SEGMENT_H__


#include <gdk/gdk.h>
#include <gtk/gtkmisc.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GTK_TYPE_SEGMENT                  (gtk_segment_get_type ())
#define GTK_SEGMENT(obj)                  (GTK_CHECK_CAST ((obj), GTK_TYPE_SEGMENT, GtkSegment))
#define GTK_SEGMENT_CLASS(klass)          (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_SEGMENT, GtkSegmentClass))
#define GTK_IS_SEGMENT(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_SEGMENT))
#define GTK_IS_SEGMENT_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_SEGMENT))


typedef struct _GtkSegment       GtkSegment;
typedef struct _GtkSegmentClass  GtkSegmentClass;

enum {
  GTK_SEGMENT_BLACK,
  GTK_SEGMENT_RED,
  GTK_SEGMENT_GREEN,
  GTK_SEGMENT_BLUE,
  GTK_SEGMENT_YELLOW
};

#define GTK_SEGMENT_DEFAULT GTK_SEGMENT_RED

typedef gint16 GtkSegmentColor;

struct _GtkSegment
{
  GtkMisc misc;

  GtkSegmentColor segment_color;
  unsigned char segment_value;
  GdkGC *redGC;
  GdkGC *greenGC;
  GdkGC *blueGC;
  GdkGC *yellowGC;
  int height;
};

struct _GtkSegmentClass
{
  GtkMiscClass parent_class;
};


GtkType    gtk_segment_get_type(void);
GtkWidget* gtk_segment_new();
GtkWidget* gtk_segment_new_with_color(GtkSegmentColor color);
int       gtk_segment_set_color(GtkSegment *segment, GtkSegmentColor color);
int       gtk_segment_set_value(GtkSegment *segment, unsigned char value);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GTK_SEGMENT_H__ */


