/*
 * A test program to exercise the GtkSegment widget
 * Copyright (C) 2001 Syd Logan
 * Original Code: Syd Logan 9/21/2001
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include "gtkticker.h"
#include "gtksegment.h"

static GtkWidget *
CreatePixmap(char *file, GdkWindow *window, GdkColor *background)
{
  GdkPixmap *pmap;
  GdkBitmap *mask;
  GtkWidget *wpmap;

  pmap = gdk_pixmap_create_from_xpm(window, &mask, background, file);
  wpmap = gtk_pixmap_new(pmap, mask);
  gtk_pixmap_set_build_insensitive(GTK_PIXMAP(wpmap), 1);
  return( wpmap );
}

void
DisplayData( GtkWidget *widget, GdkEventButton *event, gpointer arg )
{
  GtkWidget *dialog, *label, *button;
  char *name = (char *) arg;
  char buf[128] = "undefined";

  if ( event->button == 1 ) {
    dialog = gtk_dialog_new();
    gtk_window_set_title( GTK_WINDOW( dialog ), "Device Status" );
    gtk_widget_set_usize( dialog, 214, 117 );
    gtk_window_set_position( GTK_WINDOW( dialog ), GTK_WIN_POS_MOUSE );
    gtk_window_set_policy( GTK_WINDOW(dialog), FALSE, FALSE, FALSE );
    if ( !strcmp( name, "temp1" ) )
      sprintf( buf, "Valve 'a' in main area\nTemperature 82.7\nStatus Green" );
    else if ( !strcmp( name, "temp2" ) )
      sprintf( buf, "Valve 'b' in main area\nTemperature 102.1\nStatus Red" );
    else if ( !strcmp( name, "temp3" ) )
      sprintf( buf, "Conductor main circuit\nTemperature 100.3\nStatus Red" );
    else if ( !strcmp( name, "temp4" ) )
      sprintf( buf, "Exhaust area 'a'\nTemperature 76.5\nStatus Green" );
    else if ( !strcmp( name, "temp5" ) )
      sprintf( buf, "Exhaust area 'b'\nTemperature 89.3.1\nStatus Yellow" );
    label = gtk_label_new( buf );
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->vbox), label,
      FALSE, FALSE, 10 );
    button = gtk_button_new_with_label("OK");
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_box_pack_start( GTK_BOX(GTK_DIALOG(dialog)->action_area), button,
      FALSE, FALSE, 0 );
    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
      GTK_SIGNAL_FUNC (gtk_widget_destroy), (GtkObject*) dialog);

    gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
      GTK_SIGNAL_FUNC(gtk_widget_destroyed), NULL);
    gtk_signal_connect (GTK_OBJECT (dialog), "delete-event",
      GTK_SIGNAL_FUNC (gtk_false), NULL);

    gtk_widget_show_all( dialog );
    gtk_widget_grab_default (button);
  }
}

GtkWidget *
CreateTemp1( GtkWidget *widget )
{
  GtkWidget *event, *box, *digit, *pixmap;

  box = gtk_hbox_new( FALSE, 0 );
  
  pixmap = CreatePixmap( "bluevalve.xpm", 
    widget->window, &widget->style->bg[GTK_STATE_NORMAL] );
  event = gtk_event_box_new();
  gtk_container_add( GTK_CONTAINER( event ), pixmap );
  gtk_widget_set_events( event, GDK_BUTTON_PRESS_MASK );
  gtk_signal_connect( GTK_OBJECT( event ), "button_press_event", 
    (GtkSignalFunc) DisplayData, "temp1" );
  gtk_box_pack_start( GTK_BOX(box), event, FALSE, FALSE, 10);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_GREEN);
  gtk_segment_set_value(GTK_SEGMENT(digit), '8');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_GREEN);
  gtk_segment_set_value(GTK_SEGMENT(digit), '2');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_GREEN);
  gtk_segment_set_value(GTK_SEGMENT(digit), '.');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_GREEN);
  gtk_segment_set_value(GTK_SEGMENT(digit), '7');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);

  return ( box );
}

GtkWidget *
CreateTemp2( GtkWidget *widget )
{
  GtkWidget *event, *box, *digit, *pixmap;

  box = gtk_hbox_new( FALSE, 0 );
  
  pixmap = CreatePixmap( "greenvalve.xpm", 
    widget->window, &widget->style->bg[GTK_STATE_NORMAL] );
  event = gtk_event_box_new();
  gtk_container_add( GTK_CONTAINER( event ), pixmap );
  gtk_widget_set_events( event, GDK_BUTTON_PRESS_MASK );
  gtk_signal_connect( GTK_OBJECT( event ), "button_press_event", 
    (GtkSignalFunc) DisplayData, "temp2" );
  gtk_box_pack_start( GTK_BOX(box), event, FALSE, FALSE, 10);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '1');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '0');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '2');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '.');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '1');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);

  return ( box );
}

GtkWidget *
CreateTemp3( GtkWidget *widget )
{
  GtkWidget *event, *box, *digit, *pixmap;

  box = gtk_hbox_new( FALSE, 0 );
 
  pixmap = CreatePixmap( "greenvalve.xpm", 
    widget->window, &widget->style->bg[GTK_STATE_NORMAL] );
  event = gtk_event_box_new();
  gtk_container_add( GTK_CONTAINER( event ), pixmap );
  gtk_widget_set_events( event, GDK_BUTTON_PRESS_MASK );
  gtk_signal_connect( GTK_OBJECT( event ), "button_press_event", 
    (GtkSignalFunc) DisplayData, "temp3" );
  gtk_box_pack_start( GTK_BOX(box), event, FALSE, FALSE, 10);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '1');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '0');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '0');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '.');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_RED);
  gtk_segment_set_value(GTK_SEGMENT(digit), '3');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);

  return ( box );
}

GtkWidget *
CreateTemp4( GtkWidget *widget )
{
  GtkWidget *event, *box, *digit, *pixmap;

  box = gtk_hbox_new( FALSE, 0 );
  
  pixmap = CreatePixmap( "redvalve.xpm", 
    widget->window, &widget->style->bg[GTK_STATE_NORMAL] );
  event = gtk_event_box_new();
  gtk_container_add( GTK_CONTAINER( event ), pixmap );
  gtk_widget_set_events( event, GDK_BUTTON_PRESS_MASK );
  gtk_signal_connect( GTK_OBJECT( event ), "button_press_event", 
    (GtkSignalFunc) DisplayData, "temp4" );
  gtk_box_pack_start( GTK_BOX(box), event, FALSE, FALSE, 10);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_GREEN);
  gtk_segment_set_value(GTK_SEGMENT(digit), '7');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_GREEN);
  gtk_segment_set_value(GTK_SEGMENT(digit), '6');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_GREEN);
  gtk_segment_set_value(GTK_SEGMENT(digit), '.');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_GREEN);
  gtk_segment_set_value(GTK_SEGMENT(digit), '5');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);

  return ( box );
}

GtkWidget *
CreateTemp5( GtkWidget *widget )
{
  GtkWidget *event, *box, *digit, *pixmap;

  box = gtk_hbox_new( FALSE, 0 );
  
  pixmap = CreatePixmap( "purplevalve.xpm", 
    widget->window, &widget->style->bg[GTK_STATE_NORMAL] );
  event = gtk_event_box_new();
  gtk_container_add( GTK_CONTAINER( event ), pixmap );
  gtk_widget_set_events( event, GDK_BUTTON_PRESS_MASK );
  gtk_signal_connect( GTK_OBJECT( event ), "button_press_event", 
    (GtkSignalFunc) DisplayData, "temp5" );
  gtk_box_pack_start( GTK_BOX(box), event, FALSE, FALSE, 10);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_YELLOW);
  gtk_segment_set_value(GTK_SEGMENT(digit), '8');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_YELLOW);
  gtk_segment_set_value(GTK_SEGMENT(digit), '9');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_YELLOW);
  gtk_segment_set_value(GTK_SEGMENT(digit), '.');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);
  digit = gtk_segment_new();
  gtk_segment_set_color(GTK_SEGMENT(digit), GTK_SEGMENT_YELLOW);
  gtk_segment_set_value(GTK_SEGMENT(digit), '3');
  gtk_box_pack_start_defaults( GTK_BOX(box), digit);

  return ( box );
}

main( int argc, char *argv[] )
{
  GtkWidget *window, *ticker, *temp;
  int i;

  gtk_init( &argc, &argv );

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_position( GTK_WINDOW( window ), GTK_WIN_POS_MOUSE );

  gtk_signal_connect (GTK_OBJECT (window), "destroy",
    GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
  gtk_signal_connect (GTK_OBJECT (window), "delete-event",
    GTK_SIGNAL_FUNC (gtk_false), NULL);

  ticker = gtk_ticker_new();
  gtk_container_add( GTK_CONTAINER( window ), ticker );
  gtk_ticker_set_spacing( GTK_TICKER( ticker ), 20 );
  gtk_widget_set_usize( ticker, 100, -1 );

  gtk_ticker_set_interval( GTK_TICKER( ticker ), 200 );
  gtk_ticker_set_scootch( GTK_TICKER( ticker ), 10 );
  gtk_ticker_start_scroll( GTK_TICKER( ticker ) );

  gtk_widget_show_all( window );

  temp = CreateTemp1( window );
  gtk_ticker_add( GTK_TICKER( ticker ), temp );

  temp = CreateTemp2( window );
  gtk_ticker_add( GTK_TICKER( ticker ), temp );

  temp = CreateTemp3( window );
  gtk_ticker_add( GTK_TICKER( ticker ), temp );

  temp = CreateTemp4( window );
  gtk_ticker_add( GTK_TICKER( ticker ), temp );

  temp = CreateTemp5( window );
  gtk_ticker_add( GTK_TICKER( ticker ), temp );

  gtk_widget_show_all( window );

  gtk_main();
}
